@@ -11,8 +11,10 @@
 
 """Module that contains the command line application."""
 
+from __future__ import annotations
+
 import argparse
-from typing import List, Optional, Sequence
+from typing import Any, Sequence
 
 from failprint.capture import Capture
 from failprint.formats import accept_custom_format, formats
@@ -28,7 +30,7 @@ class ArgParser(argparse.ArgumentParser):
         falsy: Sequence[str],
         truthy_help: str = "",
         falsy_help: str = "",
-        **kwargs,
+        **kwargs: Any,
     ) -> None:
         """
         Add a boolean flag/argument to the parser.
@@ -48,7 +50,7 @@ class ArgParser(argparse.ArgumentParser):
         mxg.add_argument(*falsy, **falsy_kwargs)  # type: ignore
 
 
-def add_flags(parser, set_defaults=True) -> ArgParser:
+def add_flags(parser: ArgParser, set_defaults: bool = True) -> ArgParser:
     """
     Add some boolean flags to the parser.
 
@@ -105,6 +107,7 @@ def add_flags(parser, set_defaults=True) -> ArgParser:
         truthy_help="Print progress while running a command.",
         falsy_help="Don't print progress while running a command.",
     )
+    # TODO: specific to the format
     parser.add_bool_argument(
         ["-q", "--quiet"],
         ["-Q", "--no-quiet"],
@@ -113,6 +116,7 @@ def add_flags(parser, set_defaults=True) -> ArgParser:
         truthy_help="Don't print the command output, even if it failed.",
         falsy_help="Print the command output when it fails.",
     )
+    # TODO: specific to the format
     parser.add_bool_argument(
         ["-s", "--silent"],
         ["-S", "--no-silent"],
@@ -133,26 +137,26 @@ def add_flags(parser, set_defaults=True) -> ArgParser:
 
 
 def get_parser() -> ArgParser:
-    """
-    Return the CLI argument parser.
+    """Return the CLI argument parser.
 
     Returns:
         An argparse parser.
     """
     parser = add_flags(ArgParser(prog="failprint"))
+    # TODO: specific to the format
     parser.add_argument("-n", "--number", type=int, default=1, help="Command number. Useful for the 'tap' format.")
+    # TODO: specific to the format
     parser.add_argument("-t", "--title", help="Command title. Default is the command itself.")
     parser.add_argument("cmd", metavar="COMMAND", nargs="+")
     return parser
 
 
-def main(args: Optional[List[str]] = None) -> int:
-    """
-    Run the main program.
+def main(args: list[str] | None = None) -> int:
+    """Run the main program.
 
     This function is executed when you type `failprint` or `python -m failprint`.
 
-    Arguments:
+    Parameters:
         args: Arguments passed from the command line.
 
     Returns:
