@@ -1,7 +1,11 @@
 """Output-printing formats."""
 
-from typing import Callable, Dict, List, Optional
+from __future__ import annotations
 
+import inspect
+from typing import Callable, Sequence
+
+from failprint.lazy import LazyCallable
 from failprint.types import CmdFuncType
 
 DEFAULT_FORMAT = "pretty"
@@ -10,7 +14,7 @@ DEFAULT_FORMAT = "pretty"
 class Format:
     """Class to define a display format."""
 
-    def __init__(self, template: str, progress_template: Optional[str] = None, accept_ansi: bool = True) -> None:
+    def __init__(self, template: str, progress_template: str | None = None, accept_ansi: bool = True) -> None:
         """
         Initialize the object.
 
@@ -24,7 +28,7 @@ class Format:
         self.accept_ansi = accept_ansi
 
 
-formats: Dict[str, Format] = {
+formats: dict[str, Format] = {
     "pretty": Format(
         "{% if success %}<green>✓</green>"
         "{% elif nofail %}<yellow>✗</yellow>"
@@ -32,14 +36,14 @@ formats: Dict[str, Format] = {
         "<bold>{{ title or command }}</bold>"
         "{% if failure %} ({{ code }}){% endif %}"
         "{% if failure and output and not quiet %}\n"
-        "{{ ('  > ' + command + '\n') if title else '' }}"
+        "{{ ('  > ' + command + '\n') if title and command else '' }}"
         "{{ output|indent(2 * ' ') }}{% endif %}",
         progress_template="> {{ title or command }}",
     ),
     "tap": Format(
         "{% if failure %}not {% endif %}ok {{ number }} - {{ title or command }}"
         "{% if failure and output %}\n  ---\n  "
-        "{{ ('command: ' + command + '\n  ') if title else '' }}"
+        "{{ ('command: ' + command + '\n  ') if title and command else '' }}"
         "output: |\n{{ output|indent(4 * ' ') }}\n  ...{% endif %}",
         accept_ansi=False,
     ),
@@ -62,7 +66,7 @@ def accept_custom_format(string: str) -> str:
     return string
 
 
-def printable_command(cmd: CmdFuncType, args=None, kwargs=None) -> str:
+def printable_command(cmd: CmdFuncType, args: Sequence | None = None, kwargs: dict | None = None) -> str:
     """
     Transform a command or function into a string.
 
@@ -76,12 +80,14 @@ def printable_command(cmd: CmdFuncType, args=None, kwargs=None) -> str:
     """
     if isinstance(cmd, str):
         return cmd
+    if isinstance(cmd, LazyCallable):
+        return as_python_statement(cmd.call, cmd.args, cmd.kwargs)
     if callable(cmd):
         return as_python_statement(cmd, args, kwargs)
     return as_shell_command(cmd)
 
 
-def as_shell_command(cmd: List[str]) -> str:  # noqa: WPS231 (not that complex)
+def as_shell_command(cmd: list[str]) -> str:  # noqa: WPS231 (not that complex)
     """
     Rebuild a command line from system arguments.
 
@@ -116,7 +122,7 @@ def as_shell_command(cmd: List[str]) -> str:  # noqa: WPS231 (not that complex)
     return " ".join(parts)
 
 
-def as_python_statement(func: Callable, args=None, kwargs=None) -> str:
+def as_python_statement(func: Callable, args: Sequence | None = None, kwargs: dict | None = None) -> str:
     """
     Transform a callable and its arguments into a Python statement string.
 
@@ -128,7 +134,17 @@ def as_python_statement(func: Callable, args=None, kwargs=None) -> str:
     Returns:
         A Python statement.
     """
-    args = [repr(arg) for arg in args] if args else []
-    kwargs = [f"{k}={v!r}" for k, v in kwargs.items()] if kwargs else []  # noqa: WPS111,WPS221 (short name, complexity)
-    args_str = ", ".join(args + kwargs)
-    return f"{func.__name__}({args_str})"
+    func_name = getattr(func, "__name__", None)
+    try:  # noqa: WPS229
+        # climb back up to the frame above the call to run(),
+        # to get the name passed from the external caller (user)
+        ctx_run_call = inspect.currentframe().f_back.f_back.f_back.f_back  # type: ignore[union-attr]
+        call_vars = ctx_run_call.f_locals.items()  # type: ignore[union-attr]
+        func_name = next(var_name for var_name, var_val in call_vars if var_val is func)
+    except (AttributeError, StopIteration):
+        func_name = getattr(func, "__name__", "callable")
+
+    args_str = [repr(arg) for arg in args] if args else []
+    kwargs_str = [f"{k}={v!r}" for k, v in kwargs.items()] if kwargs else []  # noqa: WPS111,WPS221
+    arguments = ", ".join(args_str + kwargs_str)
+    return f"{func_name}({arguments})"
