@@ -1,77 +1,29 @@
 """Functions to run commands and capture output."""
 
+from __future__ import annotations
+
 import os
 import shutil
 import sys
 import textwrap
 import traceback
-from contextlib import contextmanager
-from io import StringIO
-from typing import Callable, Optional, Tuple, Union
+from typing import Callable, Sequence, TextIO
 
 import colorama
 from ansimarkup import ansiprint
 from jinja2 import Environment
 
 from failprint import WINDOWS
-from failprint.capture import Capture, cast_capture
+from failprint.capture import Capture, cast_capture, stdbuffer
 from failprint.formats import DEFAULT_FORMAT, accept_custom_format, formats, printable_command
 from failprint.process import run_pty_subprocess, run_subprocess
 from failprint.types import CmdFuncType, CmdType
+from failprint.lazy import LazyCallable
 
 if WINDOWS:
     colorama.init()
 
 
-class StdBuffer:
-    """A simple placeholder for three memory buffers."""
-
-    def __init__(self, stdinput=None, stdout=None, stderr=None):
-        """
-        Initialize the object.
-
-        Arguments:
-            stdinput: String to use as standard input.
-            stdout: A buffer for standard output.
-            stderr: A buffer for standard error.
-        """
-        self.stdin = StringIO(stdinput) if stdinput is not None else sys.stdin
-        self.stdout = stdout or StringIO()
-        self.stderr = stderr or StringIO()
-
-
-@contextmanager
-def stdbuffer(stdinput=None):
-    """
-    Capture output in a `with` statement.
-
-    Arguments:
-        stdinput: String to use as standard input.
-
-    Yields:
-        An instance of `StdBuffer`.
-    """
-    old_stdin = sys.stdin
-    old_stdout = sys.stdout
-    old_stderr = sys.stderr
-
-    buffer = StdBuffer(stdinput)
-
-    sys.stdin = buffer.stdin
-    sys.stdout = buffer.stdout
-    sys.stderr = buffer.stderr
-
-    yield buffer
-
-    sys.stdin = old_stdin
-    sys.stdout = old_stdout
-    sys.stderr = old_stderr
-
-    buffer.stdin.close()
-    buffer.stdout.close()
-    buffer.stderr.close()
-
-
 class RunResult:
     """Placeholder for a run result."""
 
@@ -89,18 +41,19 @@ class RunResult:
 
 def run(  # noqa: WPS231 (high complexity)
     cmd: CmdFuncType,
-    args=None,
-    kwargs=None,
+    args: Sequence | None = None,
+    kwargs: dict | None = None,
     number: int = 1,
-    capture: Optional[Union[str, bool, Capture]] = None,
-    title: Optional[str] = None,
-    fmt: Optional[str] = None,
+    capture: str | bool | Capture | None = None,
+    title: str | None = None,
+    fmt: str | None = None,
     pty: bool = False,
     progress: bool = True,
     nofail: bool = False,
     quiet: bool = False,
     silent: bool = False,
-    stdin: Optional[str] = None,
+    stdin: str | None = None,
+    command: str | None = None,
 ) -> RunResult:
     """
     Run a command in a subprocess or a Python function, and print its output if it fails.
@@ -119,6 +72,7 @@ def run(  # noqa: WPS231 (high complexity)
         quiet: Whether to not print the command output.
         silent: Don't print anything.
         stdin: String to use as standard input.
+        command: The command to display.
 
     Returns:
         The command exit code, or 0 if `nofail` is True.
@@ -130,7 +84,7 @@ def run(  # noqa: WPS231 (high complexity)
     env = Environment(autoescape=False)  # noqa: S701 (no HTML: no need to escape)
     env.filters["indent"] = textwrap.indent
 
-    command = printable_command(cmd, args, kwargs)
+    command = command if command is not None else printable_command(cmd, args, kwargs)
 
     if not silent and progress and format_obj.progress_template:
         progress_template = env.from_string(format_obj.progress_template)
@@ -170,8 +124,8 @@ def run_command(
     capture: Capture = Capture.BOTH,
     ansi: bool = False,
     pty: bool = False,
-    stdin: Optional[str] = None,
-) -> Tuple[int, str]:
+    stdin: str | None = None,
+) -> tuple[int, str]:
     """
     Run a command.
 
@@ -208,12 +162,12 @@ def run_command(
 
 
 def run_function(
-    func,
-    args=None,
-    kwargs=None,
+    func: Callable,
+    args: Sequence | None = None,
+    kwargs: dict | None = None,
     capture: Capture = Capture.BOTH,
-    stdin: Optional[str] = None,
-) -> Tuple[int, str]:
+    stdin: str | None = None,
+) -> tuple[int, str]:
     """
     Run a function.
 
@@ -250,7 +204,12 @@ def run_function(
     return code, output
 
 
-def run_function_get_code(func: Callable, stderr, args=None, kwargs=None) -> int:  # noqa: WPS212 (return statements)
+def run_function_get_code(  # noqa: WPS212,WPS231
+    func: Callable,
+    stderr: TextIO,
+    args: Sequence,
+    kwargs: dict,
+) -> int:
     """
     Run a function and return a exit code.
 
@@ -265,10 +224,21 @@ def run_function_get_code(func: Callable, stderr, args=None, kwargs=None) -> int
     """
     try:
         result = func(*args, **kwargs)
+    except SystemExit as exit:
+        if exit.code is None:
+            return 0
+        if isinstance(exit.code, int):
+            return exit.code
+        stderr.write(str(exit.code))
+        return 1
     except Exception:  # noqa: W0703 (catching Exception on purpose)
         stderr.write(traceback.format_exc() + "\n")
         return 1
 
+    # if func was a lazy callable, recurse
+    if isinstance(result, LazyCallable):
+        return run_function_get_code(result, stderr, (), {})
+
     # first check True and False
     # because int(True) == 1 and int(False) == 0
     if result is True:
