@@ -12,10 +12,9 @@ def test_fail_without_arguments():
 
 
 def test_show_help(capsys):
-    """
-    Show help.
+    """Show help.
 
-    Arguments:
+    Parameters:
         capsys: Pytest fixture to capture output.
     """
     with pytest.raises(SystemExit):
